#       pkg_desc.bash
#       This file is part of zenpakt

#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$DOC
==========
 pkg_desc
==========

Synopsis
--------

pkg_desc package compression basename name csize usize full_desc desc [ md5sum ]

Description
-----------

get description, compressed and uncompressed size and md5sum (if $MD5SUM is non-
zero) for the package which should be uncompressed using compression utility.The
basename and name of package are used for grepping description.

The variables specified as arguments store resulting values:

csize  compressed size
usize  uncompressed size
full_desc  desc along with basename or name
desc  desc without basename/name
md5sum  the md5sum

$ENDDOC

$$

pkg_desc() {
  local _pkg_desc_size _pkg_desc_usize _pkg_desc_ _pkg_desc_full

  #compressed size of the package
  _pkg_desc_csize=$(wc -c < "$1" ) || _pkg_desc_csize=

  printf -v "$5" '%s' "$_pkg_desc_csize"

  #uncompressed size of the package. xz provides a parseable output which we can
  #use to find out the uncompressed size. Gzip also provides an option but i will
  #add it once i have confirmed the output has not changed between versions.Lzma
  #stores uncompressed size only for non-streamed files, makepkg creates streamed
  #ones so trying to read lzma to find the uncompressed size would not be helpful.
  #bzip2 doesnot store the uncompressed size.
  case $2 in
    xz)
      #XZ_DEFAULTS and XZ_OPT may mess with number of rows and columns
      #we dont need memory limitation anyway because we are not decompressing
      if _pkg_desc_usize=$(XZ_DEFAULTS= XZ_OPT= xz --robot -l "$1" ); then
        #We expect only three lines first is name, second is file and the third is
        #totals (last line)
        #FIX ME: check the number of lines, more than 3, menas more than one name
        #and file lines which means it is concatenated xz, error out because thats
        #not what we expect in a package.
        _pkg_desc_usize=${_pkg_desc_usize##*$'\n'}
        if [[ $_pkg_desc_usize = $'totals\t'* ]] ; then
          read -r _ _ _ _ _pkg_desc_usize _ <<<"$_pkg_desc_usize"
          [[ $_pkg_desc_usize = *[!0-9]* ]] && _pkg_desc_usize=
        else
          _pkg_desc_usize=
        fi
      else
        _pkg_desc_usize=
      fi
      ;;

    gzip)
      #GNU gzip from version 1.2 has supported --list option.I couldn't find any
      #versions less than 1.2 to test on ftp.gnu.org.We hope that other
      #implementations work well. Considering where zenpakt will be used, we will
      #most probably be working with GNU gzip >= 1.2. The second column of last line
      #is the compressed size.
      #FIX ME:check the number of lines, more than 2 means concatenated gz
      if _pkg_desc_usize=$(gzip -l "$1") ; then
        _pkg_desc_usize=${_pkg_desc_usize##*$'\n'}
        read -r _ _pkg_desc_usize _ <<<"$_pkg_desc_usize"
        [[ $_pkg_desc_usize = *[!0-9]* ]] && _pkg_desc_usize=
      else
        _pkg_desc_usize=
      fi
      ;;
  esac

  [[ $_pkg_desc_usize ]] ||
  _pkg_desc_usize=$( "$2" -dc "$1" | wc -c ) 2>/dev/null ||
  _pkg_desc_usize=

  printf -v "$6" '%s' "$_pkg_desc_usize"

  #First check for .txt file next to the package:
  if [[ -r ${1%.*}.txt ]] && {
    grep -q "^$3:" "${1%.*}.txt" ||
    grep -q "^$4:" "${1%.*}.txt"
  };then
    _pkg_desc_=$(< "${1%.*}.txt")

  elif ! _pkg_desc_=$("$2" -dc "$1" | tar -xOf - install/slack-desc 2>/dev/null);then
    printf -v "$7" '%s'
    printf -v "$8" '%s\n' "<no description found>"
    return 0
  fi

  _pkg_desc_full=$( echo "$_pkg_desc_" | grep "^$3:")
  _pkg_desc_full+=$( echo "$_pkg_desc_" | grep "^$4:")
  printf -v "$7" '%s' "$_pkg_desc_full"

  _pkg_desc_full=${_pkg_desc_full#$3:}
  _pkg_desc_full=${_pkg_desc_full#$4:}
  _pkg_desc_full=${_pkg_desc_full//$'\n'$3:/$'\n'}
  _pkg_desc_full=${_pkg_desc_full//$'\n'$4:/$'\n'}

  printf -v "$8" '%s' "$_pkg_desc_full"
  (( MD5SUM )) &&  read "$9" _ <<< "$(md5sum "$1" )"
  return 0
}

pkg_desc_print_define() {
  if [[ $1 = long ]] ; then
    pkg_desc_print() {
      printf '\n%s\n%s\n%s\n%s\n' \
       "${TXT_bold}PACKAGE: ${TXT_reset}$1" \
       "${TXT_bold}PACKAGE DESCRIPTION:${TXT_reset}" \
       "$2" \
       "${TXT_bold}SIZE:${TXT_reset}"
      printf 'Compressed:' ; size_b2h "$3"
      printf 'uncompressed:' ; size_b2h "$4"
    }
  elif [[ $1 = short ]] ; then
    pkg_desc_print() {
      local _pkg_desc_print_short
      size_b2h "$3" _pkg_desc_print_short
      set -- "$1: ${2%%$'\n'*}" "$_pkg_desc_print_short"
      printf '%s [%s]\n' "$@"
    }
  else
    pkg_desc_print() { : ;}
  fi
}

pkg_desc_print_define long
